@@ -8,6 +8,7 @@ import shlex
 import signal
 import subprocess
 import sys
+from pathlib import Path
 from types import FrameType
 from typing import TYPE_CHECKING, Mapping, NamedTuple, Sequence, cast
 
@@ -17,12 +18,11 @@ from pdm import termui
 from pdm.cli.commands.base import BaseCommand
 from pdm.cli.hooks import HookManager
 from pdm.cli.options import skip_option, venv_option
-from pdm.cli.utils import check_project_file, get_pep582_path
-from pdm.compat import sh_join
+from pdm.cli.utils import check_project_file
 from pdm.exceptions import PdmUsageError
 from pdm.project import Project
 from pdm.signals import pdm_signals
-from pdm.utils import is_path_relative_to
+from pdm.utils import expand_env_vars, is_path_relative_to
 
 if TYPE_CHECKING:
     from typing import Any, Callable, Iterator, TypedDict
@@ -34,28 +34,49 @@ if TYPE_CHECKING:
         env: Mapping[str, str]
         env_file: EnvFileOptions | str | None
         help: str
+        keep_going: bool
         site_packages: bool
+        working_dir: str
 
 
 def exec_opts(*options: TaskOptions | None) -> dict[str, Any]:
     return dict(
-        env={k: v for opts in options if opts for k, v in opts.get("env", {}).items()},
+        env={k: v for opts in options if opts for k, v in opts.get("env", {}).items()} or None,
         **{k: v for opts in options if opts for k, v in opts.items() if k not in ("env", "help")},
     )
 
 
-RE_ARGS_PLACEHOLDER = re.compile(r"{args(?::(?P<default>[^}]*))?}")
+RE_ARGS_PLACEHOLDER = re.compile(r"\{args(?::(?P<default>[^}]*))?\}")
+RE_PDM_PLACEHOLDER = re.compile(r"\{pdm\}")
+
+
+def _interpolate_args(script: str, args: Sequence[str]) -> tuple[str, bool]:
+    """Interpolate the `{args:[defaults]} placeholder in a string"""
+    import shlex
+
+    def replace(m: re.Match[str]) -> str:
+        default = m.group("default") or ""
+        return shlex.join(args) if args else default
+
+    interpolated, count = RE_ARGS_PLACEHOLDER.subn(replace, script)
+    return interpolated, count > 0
+
+
+def _interpolate_pdm(script: str) -> str:
+    """Interpolate the `{pdm} placeholder in a string"""
+    executable_path = Path(sys.executable)
+    pdm_executable = shlex.join([executable_path.as_posix(), "-m", "pdm"])
+
+    interpolated = RE_PDM_PLACEHOLDER.sub(pdm_executable, script)
+    return interpolated
 
 
 def interpolate(script: str, args: Sequence[str]) -> tuple[str, bool]:
     """Interpolate the `{args:[defaults]} placeholder in a string"""
 
-    def replace(m: re.Match[str]) -> str:
-        default = m.group("default") or ""
-        return sh_join(args) if args else default
-
-    interpolated, count = RE_ARGS_PLACEHOLDER.subn(replace, script)
-    return interpolated, count > 0
+    script, args_interpolated = _interpolate_args(script, args)
+    script = _interpolate_pdm(script)
+    return script, args_interpolated
 
 
 class Task(NamedTuple):
@@ -84,7 +105,7 @@ class TaskRunner:
     """The task runner for pdm project"""
 
     TYPES = ("cmd", "shell", "call", "composite")
-    OPTIONS = ("env", "env_file", "help", "site_packages")
+    OPTIONS = ("env", "env_file", "help", "keep_going", "working_dir", "site_packages")
 
     def __init__(self, project: Project, hooks: HookManager) -> None:
         self.project = project
@@ -120,7 +141,7 @@ class TaskRunner:
         return Task(kind, script_name, value, cast("TaskOptions", options))
 
     def expand_command(self, command: str) -> str:
-        expanded_command = os.path.expanduser(os.path.expandvars(command))
+        expanded_command = os.path.expanduser(command)
         if expanded_command.replace(os.sep, "/").startswith(("./", "../")):
             abspath = os.path.abspath(expanded_command)
             if not os.path.isfile(abspath):
@@ -139,6 +160,7 @@ class TaskRunner:
         site_packages: bool = False,
         env: Mapping[str, str] | None = None,
         env_file: EnvFileOptions | str | None = None,
+        working_dir: str | None = None,
     ) -> int:
         """Run command in a subprocess and return the exit code."""
         project = self.project
@@ -163,8 +185,6 @@ class TaskRunner:
                 process_env = {**process_env, **dotenv_env}
             else:
                 process_env = {**dotenv_env, **process_env}
-        pythonpath = process_env.get("PYTHONPATH", "").split(os.pathsep)
-        pythonpath = [get_pep582_path(project)] + [p for p in pythonpath if "pdm/pep582" not in p.replace("\\", "/")]
         project_env = project.environment
         this_path = project_env.get_paths()["scripts"]
         process_env.update(project_env.process_env)
@@ -172,16 +192,17 @@ class TaskRunner:
             process_env.update(env)
         if shell:
             assert isinstance(args, str)
-            expanded_args: str | Sequence[str] = os.path.expandvars(args)
+            # environment variables will be expanded by shell
+            process_cmd: str | Sequence[str] = args
         else:
             assert isinstance(args, Sequence)
-            command, *args = args
+            command, *args = (expand_env_vars(arg, env=process_env) for arg in args)
             if command.endswith(".py"):
                 args = [command, *args]
                 command = str(project.environment.interpreter.executable)
             expanded_command = self.expand_command(command)
             real_command = os.path.realpath(expanded_command)
-            expanded_args = [os.path.expandvars(arg) for arg in [expanded_command, *args]]
+            process_cmd = [expanded_command, *args]
             if (
                 project_env.is_local
                 and not site_packages
@@ -194,7 +215,7 @@ class TaskRunner:
                 # Don't load system site-packages
                 process_env["NO_SITE_PACKAGES"] = "1"
 
-        cwd = project.root if chdir else None
+        cwd = (project.root / working_dir) if working_dir else project.root if chdir else None
 
         def forward_signal(signum: int, frame: FrameType | None) -> None:
             if sys.platform == "win32" and signum == signal.SIGINT:
@@ -203,7 +224,7 @@ class TaskRunner:
 
         handle_term = signal.signal(signal.SIGTERM, forward_signal)
         handle_int = signal.signal(signal.SIGINT, forward_signal)
-        process = subprocess.Popen(expanded_args, cwd=cwd, env=process_env, shell=shell, bufsize=0)
+        process = subprocess.Popen(process_cmd, cwd=cwd, env=process_env, shell=shell, bufsize=0)
         process.wait()
         signal.signal(signal.SIGTERM, handle_term)
         signal.signal(signal.SIGINT, handle_int)
@@ -251,7 +272,9 @@ class TaskRunner:
         if kind == "composite":
             args = list(args)
             should_interpolate = any(RE_ARGS_PLACEHOLDER.search(script) for script in value)
-            code = 0
+            should_interpolate = should_interpolate or any(RE_PDM_PLACEHOLDER.search(script) for script in value)
+            composite_code = 0
+            keep_going = options.pop("keep_going", False) if options else False
             for script in value:
                 if should_interpolate:
                     script, _ = interpolate(script, args)
@@ -260,14 +283,11 @@ class TaskRunner:
                 subargs = split[1:] + ([] if should_interpolate else args)
                 code = self.run(cmd, subargs, options, chdir=True)
                 if code != 0:
-                    return code
-            return code
-        return self._run_process(
-            args,
-            chdir=True,
-            shell=shell,
-            **exec_opts(self.global_options, options, opts),
-        )
+                    if not keep_going:
+                        return code
+                    composite_code = code
+            return composite_code
+        return self._run_process(args, chdir=True, shell=shell, **exec_opts(self.global_options, options, opts))
 
     def run(self, command: str, args: list[str], opts: TaskOptions | None = None, chdir: bool = False) -> int:
         if command in self.hooks.skip:
@@ -289,11 +309,7 @@ class TaskRunner:
             self.hooks.try_emit("post_script", script=command, args=args)
             return code
         else:
-            return self._run_process(
-                [command, *args],
-                chdir=chdir,
-                **exec_opts(self.global_options, opts),
-            )
+            return self._run_process([command, *args], chdir=chdir, **exec_opts(self.global_options, opts))
 
     def show_list(self) -> None:
         if not self.project.scripts:
@@ -387,11 +403,7 @@ class Command(BaseCommand):
         if options.site_packages:
             runner.global_options.update({"site_packages": options.site_packages})
         if not options.script:
-            project.core.ui.echo(
-                "No command is given, default to the Python REPL.",
-                style="warning",
-                err=True,
-            )
+            project.core.ui.warn("No command is given, default to the Python REPL.")
             options.script = "python"
         hooks.try_emit("pre_run", script=options.script, args=options.args)
         exit_code = runner.run(options.script, options.args)
