@@ -9,11 +9,13 @@ from pdm.cli.options import skip_option
 from pdm.exceptions import NoPythonVersion
 from pdm.models.caches import JSONFileCache
 from pdm.models.python import PythonInfo
+from pdm.models.venv import get_venv_python
 from pdm.project import Project
+from pdm.utils import is_conda_base_python
 
 
 class Command(BaseCommand):
-    """Use the given python version or path as base interpreter"""
+    """Use the given python version or path as base interpreter. If not found, PDM will try to install one."""
 
     def add_arguments(self, parser: argparse.ArgumentParser) -> None:
         skip_option.add_to_parser(parser)
@@ -61,45 +63,34 @@ class Command(BaseCommand):
             path = use_cache.get(python)
             cached_python = PythonInfo.from_path(path)
             if not cached_python.valid:
-                project.core.ui.echo(
+                project.core.ui.error(
                     f"The last selection is corrupted. {path!r}",
-                    style="error",
-                    err=True,
                 )
             elif version_matcher(cached_python):
-                project.core.ui.echo(
-                    "Using the last selection, add '-i' to ignore it.",
-                    style="warning",
-                    err=True,
-                )
+                project.core.ui.info("Using the last selection, add '-i' to ignore it.")
                 return cached_python
 
-        found_interpreters = list(dict.fromkeys(project.find_interpreters(python)))
-        matching_interpreters = list(filter(version_matcher, found_interpreters))
+        found_interpreters = list(dict.fromkeys(project.iter_interpreters(python, filter_func=version_matcher)))
         if not found_interpreters:
-            raise NoPythonVersion(f"No Python interpreter matching [success]{python}[/] is found.")
-        if not matching_interpreters:
-            project.core.ui.echo("Interpreters found but not matching:", err=True)
-            for py in found_interpreters:
-                info = py.identifier if py.valid else "Invalid"
-                project.core.ui.echo(f"  - {py.path} ({info})", err=True)
-            raise NoPythonVersion(
-                f"No python is found meeting the requirement [success]python {project.python_requires!s}[/]"
-            )
-        if first or len(matching_interpreters) == 1:
-            return matching_interpreters[0]
+            req = python if ignore_requires_python else f'requires-python="{project.python_requires}"'
+            raise NoPythonVersion(f"No Python interpreter matching [success]{req}[/] is found.")
+
+        if first or len(found_interpreters) == 1:
+            return found_interpreters[0]
 
         project.core.ui.echo("Please enter the Python interpreter to use")
-        for i, py_version in enumerate(matching_interpreters):
-            project.core.ui.echo(f"{i}. [success]{py_version.path!s}[/] ({py_version.identifier})")
+        for i, py_version in enumerate(found_interpreters):
+            project.core.ui.echo(
+                f"{i:>2}. [success]{py_version.implementation}@{py_version.identifier}[/] ({py_version.path!s})"
+            )
         selection = termui.ask(
             "Please select",
             default="0",
             prompt_type=int,
-            choices=[str(i) for i in range(len(matching_interpreters))],
+            choices=[str(i) for i in range(len(found_interpreters))],
             show_choices=False,
         )
-        return matching_interpreters[int(selection)]
+        return found_interpreters[int(selection)]
 
     def do_use(
         self,
@@ -115,7 +106,7 @@ class Command(BaseCommand):
         """Use the specified python version and save in project config.
         The python can be a version string or interpreter path.
         """
-        from pdm.environments.local import PythonLocalEnvironment
+        from pdm.environments import PythonLocalEnvironment
 
         selected_python = self.select_python(
             project,
@@ -129,6 +120,11 @@ class Command(BaseCommand):
             use_cache: JSONFileCache[str, str] = JSONFileCache(project.cache_dir / "use_cache.json")
             use_cache.set(python, selected_python.path.as_posix())
 
+        if project.config["python.use_venv"] and (
+            selected_python.get_venv() is None or is_conda_base_python(selected_python.path)
+        ):
+            venv_path = project._create_virtualenv(str(selected_python.path))
+            selected_python = PythonInfo.from_path(get_venv_python(venv_path))
         if not save:
             return selected_python
 
@@ -139,18 +135,15 @@ class Command(BaseCommand):
         )
         project.python = selected_python
         if project.environment.is_local:
+            assert isinstance(project.environment, PythonLocalEnvironment)
             project.core.ui.echo(
                 "Using __pypackages__ because non-venv Python is used.",
                 style="primary",
                 err=True,
             )
-        if (
-            old_python
-            and old_python.executable != selected_python.executable
-            and isinstance(project.environment, PythonLocalEnvironment)
-        ):
-            project.core.ui.echo("Updating executable scripts...", style="primary")
-            project.environment.update_shebangs(selected_python.executable.as_posix())
+            if old_python and old_python.executable != selected_python.executable:
+                project.core.ui.echo("Updating executable scripts...", style="primary")
+                project.environment.update_shebangs(selected_python.executable.as_posix())
 
         hooks = hooks or HookManager(project)
         hooks.try_emit("post_use", python=selected_python)
