@@ -8,10 +8,11 @@ import shutil
 import subprocess
 import sys
 import tempfile
+import weakref
 from contextlib import contextmanager
 from functools import cached_property, partial
 from pathlib import Path
-from typing import TYPE_CHECKING, Generator, no_type_check
+from typing import TYPE_CHECKING, Generator, Mapping, no_type_check
 
 from pdm.exceptions import BuildError, PdmUsageError
 from pdm.models.in_process import get_pep508_environment, get_python_abis, get_uname, sysconfig_get_platform
@@ -21,9 +22,11 @@ from pdm.utils import deprecation_warning, get_trusted_hosts, is_pip_compatible_
 
 if TYPE_CHECKING:
     import unearth
+    from httpx import BaseTransport
+    from httpx._types import CertTypes, VerifyTypes
 
     from pdm._types import RepositoryConfig
-    from pdm.models.session import PDMSession
+    from pdm.models.session import PDMPyPIClient
     from pdm.project import Project
 
 
@@ -42,6 +45,7 @@ def get_paths_wrapper(get_paths):  # pragma: no cover
 class BaseEnvironment(abc.ABC):
     """Environment dependent stuff related to the selected Python interpreter."""
 
+    project: Project
     is_local = False
 
     def __init_subclass__(cls) -> None:
@@ -58,8 +62,11 @@ class BaseEnvironment(abc.ABC):
         """
         from pdm.models.auth import PdmBasicAuth
 
+        if isinstance(project, weakref.ProxyTypes):
+            self.project = project
+        else:
+            self.project = weakref.proxy(project)
         self.python_requires = project.python_requires
-        self.project = project
         self.auth = PdmBasicAuth(project.core.ui, self.project.sources)
         if python is None:
             self._interpreter = project.python
@@ -104,23 +111,49 @@ class BaseEnvironment(abc.ABC):
             tp.supported_tags()
         return tp
 
-    def _build_session(self, trusted_hosts: list[str]) -> PDMSession:
-        from pdm.models.session import PDMSession
+    def _build_session(
+        self,
+        trusted_hosts: list[str] | None = None,
+        verify: VerifyTypes | None = None,
+        cert: CertTypes | None = None,
+        mounts: Mapping[str, BaseTransport | None] | None = None,
+    ) -> PDMPyPIClient:
+        from pdm.models.session import PDMPyPIClient
 
-        ca_certs = self.project.config.get("pypi.ca_certs")
-        session = PDMSession(
-            cache_dir=self.project.cache("http"),
-            trusted_hosts=trusted_hosts,
-            ca_certificates=Path(ca_certs) if ca_certs is not None else None,
-        )
-        certfn = self.project.config.get("pypi.client_cert")
-        if certfn:
+        if trusted_hosts is None:
+            trusted_hosts = get_trusted_hosts(self.project.sources)
+
+        if verify is None:
+            verify = self.project.config.get("pypi.ca_certs")
+
+        if cert is None:
+            certfn = self.project.config.get("pypi.client_cert")
             keyfn = self.project.config.get("pypi.client_key")
-            session.cert = (Path(certfn), Path(keyfn) if keyfn else None)
+            if certfn:
+                cert = (certfn, keyfn)
 
-        session.auth = self.auth
+        session_args = {
+            "cache_dir": self.project.cache("http"),
+            "trusted_hosts": trusted_hosts,
+            "timeout": self.project.config["request_timeout"],
+            "auth": self.auth,
+        }
+        if verify is not None:
+            session_args["verify"] = verify
+        if cert is not None:
+            session_args["cert"] = cert
+        if mounts:
+            session_args["mounts"] = mounts
+
+        session = PDMPyPIClient(**session_args)
+        self.project.core.exit_stack.callback(session.close)
         return session
 
+    @cached_property
+    def session(self) -> PDMPyPIClient:
+        """Build the session and cache it."""
+        return self._build_session()
+
     @contextmanager
     def _patch_target_python(self) -> Generator[None, None, None]:
         """Patch the packaging modules to respect the arch of target python."""
@@ -172,11 +205,8 @@ class BaseEnvironment(abc.ABC):
                 f"{self.project.config['pypi.ignore_stored_index']}"
             )
 
-        trusted_hosts = get_trusted_hosts(sources)
-
-        session = self._build_session(trusted_hosts)
         finder = PDMPackageFinder(
-            session=session,
+            session=self.session,
             target_python=self.target_python,
             ignore_compatibility=ignore_compatibility,
             no_binary=os.getenv("PDM_NO_BINARY", "").split(","),
@@ -187,6 +217,7 @@ class BaseEnvironment(abc.ABC):
             ),
             verbosity=self.project.core.ui.verbosity,
             minimal_version=minimal_version,
+            exclude_newer_than=self.project.core.state.exclude_newer,
         )
         finder.sources.clear()
         for source in sources:
@@ -195,10 +226,7 @@ class BaseEnvironment(abc.ABC):
                 finder.add_find_links(source.url)
             else:
                 finder.add_index_url(source.url)
-        try:
-            yield finder
-        finally:
-            session.close()
+        yield finder
 
     def get_working_set(self) -> WorkingSet:
         """Get the working set based on local packages directory."""
@@ -269,6 +297,17 @@ class BaseEnvironment(abc.ABC):
             command.append("-" + "v" * verbosity)
         return command
 
+    @property
+    def script_kind(self) -> str:
+        if os.name != "nt":
+            return "posix"
+        is_32bit = self.interpreter.is_32bit
+        # TODO: support win arm64
+        if is_32bit:
+            return "win-ia32"
+        else:
+            return "win-amd64"
+
 
 class BareEnvironment(BaseEnvironment):
     """Bare environment that does not depend on project files."""
