@@ -2,7 +2,6 @@ from __future__ import annotations
 
 import dataclasses
 import functools
-import multiprocessing
 import traceback
 from concurrent.futures import Future, ThreadPoolExecutor
 from functools import cached_property
@@ -208,7 +207,6 @@ class BaseSynchronizer:
             assert can.link is not None
             return url != backend.expand_line(can.link.url_without_fragment)
         specifier = can.req.as_pinned_version(can.version).specifier
-        assert specifier is not None
         return not specifier.contains(dist.version, prereleases=True)
 
     def compare_with_working_set(self) -> tuple[list[str], list[str], list[str]]:
@@ -269,11 +267,9 @@ class BaseSynchronizer:
 
 
 class Synchronizer(BaseSynchronizer):
-    def create_executor(
-        self,
-    ) -> ThreadPoolExecutor | DummyExecutor:
+    def create_executor(self) -> ThreadPoolExecutor | DummyExecutor:
         if self.parallel:
-            return ThreadPoolExecutor(max_workers=min(multiprocessing.cpu_count(), 8))
+            return ThreadPoolExecutor()
         else:
             return DummyExecutor()
 
