@@ -5,25 +5,26 @@ import dataclasses
 import hashlib
 import json
 import os
+import stat
 from functools import lru_cache
 from pathlib import Path
-from typing import TYPE_CHECKING, BinaryIO, Generic, Iterable, TypeVar, cast
+from typing import TYPE_CHECKING, Generic, Iterable, TypeVar
 
-from cachecontrol.cache import SeparateBodyBaseCache
-from cachecontrol.caches import FileCache
+import httpx
 from packaging.utils import canonicalize_name, parse_wheel_filename
 
 from pdm._types import CandidateInfo
 from pdm.exceptions import PdmException
+from pdm.models.cached_package import CachedPackage
 from pdm.models.candidates import Candidate
 from pdm.termui import logger
-from pdm.utils import atomic_open_for_write, create_tracked_tempdir
+from pdm.utils import atomic_open_for_write, create_tracked_tempdir, get_file_hash
 
 if TYPE_CHECKING:
     from packaging.tags import Tag
-    from requests import Session
     from unearth import Link, TargetPython
 
+
 KT = TypeVar("KT")
 VT = TypeVar("VT")
 
@@ -107,7 +108,6 @@ class CandidateInfoCache(JSONFileCache[Candidate, CandidateInfo]):
 
 
 class HashCache:
-
     """Caches hashes of PyPI artifacts so we do not need to re-download them.
 
     Hashes are only cached when the URL appears to contain a hash in it and the
@@ -121,21 +121,21 @@ class HashCache:
     def __init__(self, directory: Path | str) -> None:
         self.directory = Path(directory)
 
-    def _read_from_link(self, link: Link, session: Session) -> Iterable[bytes]:
+    def _read_from_link(self, link: Link, session: httpx.Client) -> Iterable[bytes]:
         if link.is_file:
             with open(link.file_path, "rb") as f:
                 yield from f
         else:
-            import requests
+            import httpx
 
-            with session.get(link.normalized, stream=True) as resp:
+            with session.stream("GET", link.normalized) as resp:
                 try:
                     resp.raise_for_status()
-                except requests.HTTPError as e:
+                except httpx.HTTPStatusError as e:
                     raise PdmException(f"Failed to read from {link.redacted}: {e}") from e
-                yield from resp.iter_content(chunk_size=8192)
+                yield from resp.iter_bytes(chunk_size=8192)
 
-    def _get_file_hash(self, link: Link, session: Session) -> str:
+    def _get_file_hash(self, link: Link, session: httpx.Client) -> str:
         h = hashlib.new(self.FAVORITE_HASH)
         logger.debug("Downloading link %s for calculating hash", link.redacted)
         for chunk in self._read_from_link(link, session):
@@ -147,7 +147,7 @@ class HashCache:
         # We may add more when we know better about it.
         return not link.is_file
 
-    def get_hash(self, link: Link, session: Session) -> str:
+    def get_hash(self, link: Link, session: httpx.Client) -> str:
         # If there is no link hash (i.e., md5, sha256, etc.), we don't want
         # to store it.
         hash_value = self.get(link.url_without_fragment)
@@ -266,60 +266,74 @@ class WheelCache:
         return min(candidates, key=lambda x: x[0])[1]
 
 
-class SafeFileCache(SeparateBodyBaseCache):
-    """
-    A file based cache which is safe to use even when the target directory may
-    not be accessible or writable.
-    """
-
-    def __init__(self, directory: str) -> None:
-        super().__init__()
-        self.directory = directory
-
-    def _get_cache_path(self, name: str) -> str:
-        # From cachecontrol.caches.file_cache.FileCache._fn, brought into our
-        # class for backwards-compatibility and to avoid using a non-public
-        # method.
-        hashed = FileCache.encode(name)
-        parts = [*list(hashed[:5]), hashed]
-        return os.path.join(self.directory, *parts)
-
-    def get(self, key: str) -> bytes | None:
-        path = self._get_cache_path(key)
-        with contextlib.suppress(OSError):
-            with open(path, "rb") as f:
-                return f.read()
-
-        return None
-
-    def get_body(self, key: str) -> BinaryIO | None:
-        path = self._get_cache_path(key)
-        with contextlib.suppress(OSError):
-            return cast(BinaryIO, open(f"{path}.body", "rb"))
-
-        return None
-
-    def set(self, key: str, value: bytes, expires: int | None = None) -> None:
-        path = self._get_cache_path(key)
-        with contextlib.suppress(OSError):
-            with atomic_open_for_write(path, mode="wb") as f:
-                cast(BinaryIO, f).write(value)
-
-    def set_body(self, key: str, body: bytes) -> None:
-        if body is None:
-            return
-
-        path = self._get_cache_path(key)
-        with contextlib.suppress(OSError):
-            with atomic_open_for_write(f"{path}.body", mode="wb") as f:
-                cast(BinaryIO, f).write(body)
-
-    def delete(self, key: str) -> None:
-        path = self._get_cache_path(key)
-        with contextlib.suppress(OSError):
-            os.remove(path)
-
-
-@lru_cache(maxsize=128)
+@lru_cache(maxsize=None)
 def get_wheel_cache(directory: Path | str) -> WheelCache:
     return WheelCache(directory)
+
+
+class PackageCache:
+    def __init__(self, root: Path) -> None:
+        self.root = root
+
+    def cache_wheel(self, wheel: Path, checksum: str | None = None) -> CachedPackage:
+        """Create a CachedPackage instance from a wheel file"""
+        import zipfile
+
+        from installer.utils import make_file_executable
+
+        if checksum is None:
+            checksum = get_file_hash(wheel)
+        parts = (checksum[:2],)  # shard by the first two chars of the checksum
+        dest = self.root.joinpath(*parts, f"{wheel.name}.cache")
+        pkg = CachedPackage(dest, original_wheel=wheel)
+        if dest.exists():
+            return pkg
+        dest.mkdir(parents=True, exist_ok=True)
+        with pkg.lock():
+            dest.joinpath(".checksum").write_text(checksum)
+            logger.info("Unpacking wheel into cached location %s", dest)
+            with zipfile.ZipFile(wheel) as zf:
+                try:
+                    for item in zf.infolist():
+                        target_path = zf.extract(item, dest)
+                        mode = item.external_attr >> 16
+                        is_executable = bool(mode and stat.S_ISREG(mode) and mode & 0o111)
+                        if is_executable:
+                            make_file_executable(target_path)
+                except Exception:  # pragma: no cover
+                    pkg.cleanup()  # cleanup on any error
+                    raise
+        return pkg
+
+    def iter_packages(self) -> Iterable[tuple[str, CachedPackage]]:
+        for path in self.root.rglob("*.whl.cache"):
+            hash_parts = path.parent.relative_to(self.root).parts
+            p = CachedPackage(path)
+            with p.lock():  # ensure the package is not being created
+                pass
+            if not path.joinpath(".checksum").exists():
+                checksum = "".join(hash_parts)
+                path.joinpath(".checksum").write_text(checksum)
+            else:
+                checksum = p.checksum
+            yield checksum, p
+
+    def cleanup(self) -> int:
+        """Remove unused cached packages"""
+        count = 0
+        for _, pkg in self.iter_packages():
+            if not any(os.path.exists(fn) for fn in pkg.referrers):
+                pkg.cleanup()
+                count += 1
+        return count
+
+    def match_link(self, link: Link) -> CachedPackage | None:
+        """Match a link to a cached package"""
+        if not link.is_wheel:
+            return None
+        given_hash = link_hashes.get("sha256", []) if (link_hashes := link.hash_option) else []
+        for checksum, p in self.iter_packages():
+            if p.path.stem == link.filename and (not given_hash or checksum in given_hash):
+                logger.debug("Using package from cache location: %s", p.path)
+                return p
+        return None
