@@ -5,7 +5,7 @@ import hashlib
 import os
 import re
 import warnings
-from functools import lru_cache
+from functools import cached_property
 from pathlib import Path
 from tempfile import TemporaryDirectory
 from typing import TYPE_CHECKING, Any, cast, no_type_check
@@ -15,10 +15,10 @@ from packaging.utils import parse_wheel_filename
 
 from pdm import termui
 from pdm.builders import EditableBuilder, WheelBuilder
-from pdm.compat import cached_property
 from pdm.compat import importlib_metadata as im
 from pdm.exceptions import BuildError, CandidateNotFound, InvalidPyVersion, PDMWarning
 from pdm.models.backends import get_backend, get_backend_by_spec
+from pdm.models.cached_package import CachedPackage
 from pdm.models.reporter import BaseReporter
 from pdm.models.requirements import (
     FileRequirement,
@@ -31,8 +31,10 @@ from pdm.models.setup import Setup
 from pdm.models.specifiers import PySpecSet
 from pdm.utils import (
     cd,
+    comparable_version,
     convert_hashes,
-    create_tracked_tempdir,
+    filtered_sources,
+    get_file_hash,
     get_rev_from_url,
     normalize_name,
     path_to_url,
@@ -258,9 +260,12 @@ class Candidate:
     @no_type_check
     def as_lockfile_entry(self, project_root: Path) -> dict[str, Any]:
         """Build a lockfile entry dictionary for the candidate."""
+        version = str(self.version)
+        if not self.req.is_pinned:
+            version = str(comparable_version(version))
         result = {
             "name": normalize_name(self.name),
-            "version": str(self.version),
+            "version": version,
             "extras": sorted(self.req.extras or ()),
             "requires_python": str(self.requires_python),
             "editable": self.req.editable,
@@ -310,10 +315,9 @@ class PreparedCandidate:
 
     def __post_init__(self) -> None:
         self.req = self.candidate.req
-
-        self.wheel: Path | None = None
         self.link = self._replace_url_vars(self.candidate.link)
 
+        self._cached: CachedPackage | None = None
         self._source_dir: Path | None = None
         self._unpacked_dir: Path | None = None
         self._metadata_dir: str | None = None
@@ -382,28 +386,42 @@ class PreparedCandidate:
                         "subdirectory": req.subdirectory,
                     }
                 )
-            with self.environment.get_finder() as finder:
-                hash_cache = self.environment.project.make_hash_cache()
-                return _filter_none(
-                    {
-                        "url": self.link.url_without_fragment,
-                        "archive_info": {"hash": hash_cache.get_hash(self.link, finder.session).replace(":", "=")},
-                        "subdirectory": req.subdirectory,
-                    }
-                )
+            hash_cache = self.environment.project.make_hash_cache()
+            return _filter_none(
+                {
+                    "url": self.link.url_without_fragment,
+                    "archive_info": {
+                        "hash": hash_cache.get_hash(self.link, self.environment.session).replace(":", "=")
+                    },
+                    "subdirectory": req.subdirectory,
+                }
+            )
         else:
             return None
 
+    def get_cached_package(self) -> CachedPackage:
+        """Call PEP 517 build hook to build the candidate into a cached package"""
+        self._obtain(allow_all=False)
+        if self._cached:
+            return self._cached
+        wheel = self.build()
+        checksum_path = Path(f"{wheel}.sha256")
+        if checksum_path.exists():
+            checksum = checksum_path.read_text().strip()
+        else:
+            checksum = None
+        self._cached = self.environment.project.package_cache.cache_wheel(wheel, checksum)
+        return self._cached
+
     def build(self) -> Path:
         """Call PEP 517 build hook to build the candidate into a wheel"""
-        self.obtain(allow_all=False)
-        if self.wheel:
-            return self.wheel
+        self._obtain(allow_all=False, prefer_wheel=True)
+        if self._cached and self._cached.original_wheel:
+            return self._cached.original_wheel
         if not self.req.editable:
-            cached = self._get_cached_wheel()
+            cached, checksum = self._get_build_cache()
             if cached:
-                self.wheel = cached
-                return self.wheel
+                return cached
         assert self._source_dir, "Source directory isn't ready yet"
         builder_cls = EditableBuilder if self.req.editable else WheelBuilder
         builder = builder_cls(str(self._unpacked_dir), self.environment)
@@ -411,27 +429,33 @@ class PreparedCandidate:
         os.makedirs(build_dir, exist_ok=True)
         termui.logger.info("Running PEP 517 backend to build a wheel for %s", self.link)
         self.reporter.report_build_start(self.link.filename)  # type: ignore[union-attr]
-        self.wheel = Path(builder.build(build_dir, metadata_directory=self._metadata_dir))
+        wheel = Path(builder.build(build_dir, metadata_directory=self._metadata_dir))
+        checksum = get_file_hash(wheel)
+        with open(f"{wheel}.sha256", "w") as f:
+            f.write(checksum)
         self.reporter.report_build_end(self.link.filename)  # type: ignore[union-attr]
-        return self.wheel
+        return wheel
 
-    def obtain(self, allow_all: bool = False, unpack: bool = True) -> None:
+    def _obtain(self, allow_all: bool = False, unpack: bool = True, prefer_wheel: bool = False) -> None:
         """Fetch the link of the candidate and unpack to local if necessary.
 
         :param allow_all: If true, don't validate the wheel tag nor hashes
         :param unpack: Whether to download and unpack the link if it's not local
+        :param prefer_wheel: Prefer wheel over cached package
         """
-        if self.wheel:
-            if self._wheel_compatible(self.wheel.name, allow_all):
+        if self._cached and self._wheel_compatible(self._cached.path.stem, allow_all):
+            if not prefer_wheel or self._cached.original_wheel:
                 return
-        elif self._source_dir and self._source_dir.exists():
+
+        if self._source_dir and self._source_dir.exists():
             return
 
-        with self.environment.get_finder() as finder:
+        sources = filtered_sources(self.environment.project.sources, self.req.key)
+        with self.environment.get_finder(sources) as finder:
             if not self.link or self.link.is_wheel and not self._wheel_compatible(self.link.filename, allow_all):
                 if self.req.is_file_or_url:
                     raise CandidateNotFound(f"The URL requirement {self.req.as_line()} is a wheel but incompatible")
-                self.link = self.wheel = None  # reset the incompatible wheel
+                self.link = self._cached = None  # reset the incompatible wheel
                 self.link = _find_best_match_link(
                     finder,
                     self.req.as_pinned_version(self.candidate.version),
@@ -444,13 +468,21 @@ class PreparedCandidate:
                     )
                 if not self.candidate.link:
                     self.candidate.link = self.link
-            if allow_all and not self.req.editable:
-                cached = self._get_cached_wheel()
-                if cached:
-                    self.wheel = cached
-                    return
-            if unpack:
-                self._unpack(validate_hashes=not allow_all)
+        # Find if there is already a CachedPackage for the link
+        package = self.environment.project.package_cache.match_link(self.link)
+        if package is not None:
+            self._cached = package
+            if not prefer_wheel:
+                return
+        # If not, find if there is any build cache for the candidate
+        if allow_all and not self.req.editable:
+            cached, checksum = self._get_build_cache()
+            if cached:
+                self._cached = self.environment.project.package_cache.cache_wheel(cached, checksum)
+                return
+        # If not, download and unpack the link
+        if unpack:
+            self._unpack(validate_hashes=not allow_all)
 
     def _unpack(self, validate_hashes: bool = False) -> None:
         hash_options = None
@@ -472,18 +504,18 @@ class PreparedCandidate:
                     download_reporter=self.reporter.report_download,
                     unpack_reporter=self.reporter.report_unpack,
                 )
-                if self.link.is_wheel:
-                    self.wheel = result
-                else:
-                    self._source_dir = Path(build_dir)
-                    self._unpacked_dir = result
+        if self.link.is_wheel:
+            checksum = hashes["sha256"][0] if (hashes := self.link.hash_option) and "sha256" in hashes else None
+            self._cached = self.environment.project.package_cache.cache_wheel(result, checksum)
+        else:
+            self._source_dir = Path(build_dir)
+            self._unpacked_dir = result
 
     def prepare_metadata(self, force_build: bool = False) -> im.Distribution:
-        self.obtain(allow_all=True, unpack=False)
+        self._obtain(allow_all=True, unpack=False)
 
-        metadata_parent = create_tracked_tempdir(prefix="pdm-meta-")
-        if self.wheel:
-            return self._get_metadata_from_wheel(self.wheel, metadata_parent)
+        if self._cached:
+            return self._get_metadata_from_cached(self._cached)
 
         assert self.link is not None
         if self.link.dist_info_metadata:
@@ -493,8 +525,8 @@ class PreparedCandidate:
                 return dist
 
         self._unpack(validate_hashes=False)
-        if self.wheel:  # check again if the wheel is downloaded to local
-            return self._get_metadata_from_wheel(self.wheel, metadata_parent)
+        if self._cached:  # check again if the wheel is downloaded to local
+            return self._get_metadata_from_cached(self._cached)
 
         assert self._unpacked_dir, "Source directory isn't ready yet"
         pyproject_toml = self._unpacked_dir / "pyproject.toml"
@@ -504,24 +536,23 @@ class PreparedCandidate:
                 return dist
 
         # If all fail, try building the source to get the metadata
+        metadata_parent = self.environment.project.core.create_temp_dir(prefix="pdm-meta-")
         return self._get_metadata_from_build(self._unpacked_dir, metadata_parent)
 
     def _get_metadata_from_metadata_link(
         self, link: Link, medata_hash: bool | dict[str, str] | None
     ) -> im.Distribution | None:
-        with self.environment.get_finder() as finder:
-            resp = finder.session.get(link.normalized, headers={"Cache-Control": "max-age=0"})
-            if isinstance(medata_hash, dict):
-                hash_name, hash_value = next(iter(medata_hash.items()))
-                if hashlib.new(hash_name, resp.content).hexdigest() != hash_value:
-                    termui.logger.warning("Metadata hash mismatch for %s, ignoring the metadata", link)
-                    return None
-            return MetadataDistribution(resp.text)
+        resp = self.environment.session.get(link.normalized)
+        if isinstance(medata_hash, dict):
+            hash_name, hash_value = next(iter(medata_hash.items()))
+            if hashlib.new(hash_name, resp.content).hexdigest() != hash_value:
+                termui.logger.warning("Metadata hash mismatch for %s, ignoring the metadata", link)
+                return None
+        return MetadataDistribution(resp.text)
 
-    def _get_metadata_from_wheel(self, wheel: Path, metadata_parent: str) -> im.Distribution:
+    def _get_metadata_from_cached(self, cached: CachedPackage) -> im.Distribution:
         # Get metadata from METADATA inside the wheel
-        self._metadata_dir = _get_wheel_metadata_from_wheel(wheel, metadata_parent)
-        return im.PathDistribution(Path(self._metadata_dir))
+        return im.PathDistribution(cached.dist_info)
 
     def _get_metadata_from_project(self, pyproject_toml: Path) -> im.Distribution | None:
         # Try getting from PEP 621 metadata
@@ -557,10 +588,13 @@ class PreparedCandidate:
             # no variable expansion
             backend_cls = get_backend("setuptools")
         backend = backend_cls(pyproject_toml.parent)
+        if "name" not in metadata:
+            termui.logger.warning("Failed to parse pyproject.toml, name is required")
+            return None
         setup = Setup(
             name=metadata.get("name"),
             summary=metadata.get("description"),
-            version=metadata.get("version"),
+            version=metadata.get("version", "0.0.0"),
             install_requires=list(
                 map(
                     backend.expand_line,
@@ -610,11 +644,7 @@ class PreparedCandidate:
     def get_dependencies_from_metadata(self) -> list[str]:
         """Get the dependencies of a candidate from metadata."""
         extras = self.req.extras or ()
-        return filter_requirements_with_extras(
-            self.req.project_name,  # type: ignore[arg-type]
-            self.metadata.requires or [],
-            extras,
-        )
+        return filter_requirements_with_extras(self.metadata.requires or [], extras)
 
     def should_cache(self) -> bool:
         """Determine whether to cache the dependencies and built wheel."""
@@ -637,13 +667,19 @@ class PreparedCandidate:
             return _egg_info_re.search(link.filename) is not None
         return False
 
-    def _get_cached_wheel(self) -> Path | None:
+    def _get_build_cache(self) -> tuple[Path | None, str | None]:
         wheel_cache = self.environment.project.make_wheel_cache()
         assert self.candidate.link
         cache_entry = wheel_cache.get(self.candidate.link, self.candidate.name, self.environment.target_python)
         if cache_entry is not None:
             termui.logger.info("Using cached wheel: %s", cache_entry)
-        return cache_entry
+            if (hash_file := cache_entry.with_name(f"{cache_entry.name}.sha256")).exists():
+                cache_hash = hash_file.read_text().strip()
+            else:
+                cache_hash = get_file_hash(cache_entry)
+                hash_file.write_text(cache_hash)
+            return cache_entry, cache_hash
+        return cache_entry, None
 
     def _get_build_dir(self) -> str:
         original_link = self.candidate.link
@@ -671,7 +707,7 @@ class PreparedCandidate:
                 dirname, _ = os.path.splitext(original_link.filename)
             return str(src_dir / str(dirname))
         # Otherwise, for source dists, they will be unpacked into a *temp* directory.
-        return create_tracked_tempdir(prefix="pdm-build-")
+        return self.environment.project.core.create_temp_dir(prefix="pdm-build-")
 
     def _wheel_compatible(self, wheel_file: str, allow_all: bool = False) -> bool:
         if allow_all:
@@ -690,14 +726,3 @@ class PreparedCandidate:
             return wheel_cache.get_ephemeral_path_for_link(
                 self.candidate.link, self.environment.target_python
             ).as_posix()
-
-
-@lru_cache(maxsize=None)
-def make_candidate(
-    req: Requirement,
-    name: str | None = None,
-    version: str | None = None,
-    link: Link | None = None,
-) -> Candidate:
-    """Construct a candidate and cache it in memory"""
-    return Candidate(req, name, version, link)
