@@ -1,6 +1,7 @@
 """
 A collection of functions that need to be called via a subprocess call.
 """
+
 from __future__ import annotations
 
 import contextlib
@@ -20,9 +21,9 @@ def _in_process_script(name: str) -> Generator[str, None, None]:
         yield str(script)
 
 
-@functools.lru_cache()
-def get_python_abi_tag(executable: str) -> str:
-    with _in_process_script("get_abi_tag.py") as script:
+@functools.lru_cache
+def get_python_abis(executable: str) -> list[str]:
+    with _in_process_script("get_abis.py") as script:
         return json.loads(subprocess.check_output(args=[executable, "-EsS", script]))
 
 
@@ -55,8 +56,15 @@ def parse_setup_py(executable: str, path: str) -> dict[str, Any]:
             return json.load(fp)
 
 
-@functools.lru_cache()
+@functools.lru_cache
 def get_uname(executable: str) -> os.uname_result:
     """Get uname of the system"""
     script = "import os, json; print(json.dumps(os.uname()))"
     return os.uname_result(json.loads(subprocess.check_output([executable, "-EsSc", script])))
+
+
+@functools.lru_cache
+def sysconfig_get_platform(executable: str) -> str:
+    """Get platform from sysconfig"""
+    script = "import sysconfig; print(sysconfig.get_platform())"
+    return subprocess.check_output([executable, "-EsSc", script]).decode().strip()
