@@ -2,6 +2,7 @@ from __future__ import annotations
 
 import dataclasses
 import fnmatch
+import itertools
 import posixpath
 import re
 import sys
@@ -163,7 +164,7 @@ class BaseRepository:
         :param ignore_requires_python: whether to ignore the requires-python marker
         :param minimal_version: whether to prefer the minimal versions of the package
         """
-        # `allow_prereleases` is None means leave it to specifier to decide whether to
+        # `allow_prereleases` is None means to let the specifier decide whether to
         # include prereleases
         from unearth.utils import LazySequence
 
@@ -193,7 +194,7 @@ class BaseRepository:
                     if self._should_ignore_package_warning(requirement):
                         continue
                     working_requires_python = project_requires_python & PySpecSet(candidate.requires_python)
-                    if working_requires_python.is_impossible:  # pragma: no cover
+                    if working_requires_python.is_empty():  # pragma: no cover
                         continue
                     warnings.warn(
                         f"Skipping {candidate.name}@{candidate.version} because it requires "
@@ -282,11 +283,7 @@ class BaseRepository:
         extra_dependencies = project.pyproject.settings.get("dev-dependencies", {}).copy()
         extra_dependencies.update(project.pyproject.metadata.get("optional-dependencies", {}))
         if candidate.req.extras is not None:
-            reqs = sum(
-                (extra_dependencies.get(g, []) for g in candidate.req.extras),
-                [],
-            )
-
+            reqs = list(itertools.chain.from_iterable(extra_dependencies.get(g, []) for g in candidate.req.extras))
         return (
             reqs,
             str(self.environment.python_requires),
@@ -308,7 +305,7 @@ class BaseRepository:
                     spec = PySpecSet(f">={version}")  # cp37-abi3 is compatible with >=3.7
                 else:
                     spec = PySpecSet(f"~={version}")  # cp37-cp37 is only compatible with 3.7.*
-                return not (spec & python_requires).is_impossible
+                return not (spec & python_requires).is_empty()
             else:
                 # we don't know about compatility for non-cpython implementations
                 # assume it is compatible
@@ -348,28 +345,28 @@ class BaseRepository:
         if req.is_named and respect_source_order and comes_from:
             sources = [s for s in sources if comes_from.startswith(s.url)]
 
-        with self.environment.get_finder(sources, self.ignore_compatibility) as finder:
-            if req.is_file_or_url:
-                this_link = cast("Link", candidate.prepare(self.environment).link)
-                links: list[Link] = [this_link]
-            else:  # the req must be a named requirement
+        if req.is_file_or_url:
+            this_link = cast("Link", candidate.prepare(self.environment).link)
+            links: list[Link] = [this_link]
+        else:  # the req must be a named requirement
+            with self.environment.get_finder(sources, self.ignore_compatibility) as finder:
                 links = [package.link for package in finder.find_matches(req.as_line())]
-                if self.ignore_compatibility:
-                    links = [link for link in links if self._is_python_match(link)]
-            for link in links:
-                if not link or link.is_vcs or link.is_file and link.file_path.is_dir():
-                    # The links found can still be a local directory or vcs, skippping it.
-                    continue
-                if not logged:
-                    termui.logger.info("Fetching hashes for %s", candidate)
-                    logged = True
-                result.append(
-                    {
-                        "url": link.url_without_fragment,
-                        "file": link.filename,
-                        "hash": self._hash_cache.get_hash(link, finder.session),
-                    }
-                )
+            if self.ignore_compatibility:
+                links = [link for link in links if self._is_python_match(link)]
+        for link in links:
+            if not link or link.is_vcs or link.is_file and link.file_path.is_dir():
+                # The links found can still be a local directory or vcs, skippping it.
+                continue
+            if not logged:
+                termui.logger.info("Fetching hashes for %s", candidate)
+                logged = True
+            result.append(
+                {
+                    "url": link.url_without_fragment,
+                    "file": link.filename,
+                    "hash": self._hash_cache.get_hash(link, self.environment.session),
+                }
+            )
         return result
 
     def dependency_generators(self) -> Iterable[Callable[[Candidate], CandidateInfo]]:
@@ -403,24 +400,23 @@ class PyPIRepository(BaseRepository):
             for proc_url in (raw_url.rstrip("/") for raw_url in (source.url for source in sources) if raw_url)
             if proc_url.endswith("/simple")
         ]
-        with self.environment.get_finder(sources) as finder:
-            session = finder.session
-            for prefix in url_prefixes:
-                json_url = f"{prefix}/pypi/{candidate.name}/{candidate.version}/json"
-                resp = session.get(json_url)
-                if not resp.ok:
-                    continue
+        session = self.environment.session
+        for prefix in url_prefixes:
+            json_url = f"{prefix}/pypi/{candidate.name}/{candidate.version}/json"
+            resp = session.get(json_url)
+            if resp.is_error:
+                continue
 
-                info = resp.json()["info"]
+            info = resp.json()["info"]
 
-                requires_python = info["requires_python"] or ""
-                summary = info["summary"] or ""
-                try:
-                    requirement_lines = info["requires_dist"] or []
-                except KeyError:
-                    requirement_lines = info["requires"] or []
-                requirements = filter_requirements_with_extras(requirement_lines, candidate.req.extras or ())
-                return requirements, requires_python, summary
+            requires_python = info["requires_python"] or ""
+            summary = info["summary"] or ""
+            try:
+                requirement_lines = info["requires_dist"] or []
+            except KeyError:
+                requirement_lines = info["requires"] or []
+            requirements = filter_requirements_with_extras(requirement_lines, candidate.req.extras or ())
+            return requirements, requires_python, summary
         raise CandidateInfoNotFound(candidate)
 
     def dependency_generators(self) -> Iterable[Callable[[Candidate], CandidateInfo]]:
@@ -454,20 +450,19 @@ class PyPIRepository(BaseRepository):
         else:
             search_url = pypi_simple + "/search"
 
-        with self.environment.get_finder() as finder:
-            session = finder.session
-            resp = session.get(search_url, params={"q": query})
-            if resp.status_code == 404:
-                self.environment.project.core.ui.warn(
-                    f"{pypi_simple!r} doesn't support '/search' endpoint, fallback "
-                    f"to {self.DEFAULT_INDEX_URL!r} now.\n"
-                    "This may take longer depending on your network condition.",
-                )
-                resp = session.get(f"{self.DEFAULT_INDEX_URL}/search", params={"q": query})
-            parser = SearchResultParser()
-            resp.raise_for_status()
-            parser.feed(resp.text)
-            return parser.results
+        session = self.environment.session
+        resp = session.get(search_url, params={"q": query})
+        if resp.status_code == 404:
+            self.environment.project.core.ui.warn(
+                f"{pypi_simple!r} doesn't support '/search' endpoint, fallback "
+                f"to {self.DEFAULT_INDEX_URL!r} now.\n"
+                "This may take longer depending on your network condition.",
+            )
+            resp = session.get(f"{self.DEFAULT_INDEX_URL}/search", params={"q": query})
+        parser = SearchResultParser()
+        resp.raise_for_status()
+        parser.feed(resp.text)
+        return parser.results
 
 
 class LockedRepository(BaseRepository):
