@@ -1,12 +1,15 @@
 from __future__ import annotations
 
 import re
-from typing import Any, Union, overload
+from typing import TYPE_CHECKING, overload
 
-from pdm.compat import Literal
 from pdm.exceptions import InvalidPyVersion
 
-VersionBit = Union[int, Literal["*"]]  # noqa: F722
+if TYPE_CHECKING:
+    from typing import Any, Literal, Union
+
+    VersionBit = Union[int, Literal["*"]]
+
 PRE_RELEASE_SEGMENT_RE = re.compile(
     r"(?P<digit>\d+)(?P<type>a|b|rc)(?P<n>\d*)",
     flags=re.IGNORECASE,
@@ -136,12 +139,10 @@ class Version:
         return self.__gt__(other) or self.__eq__(other)
 
     @overload
-    def __getitem__(self, idx: int) -> VersionBit:
-        ...
+    def __getitem__(self, idx: int) -> VersionBit: ...
 
     @overload
-    def __getitem__(self, idx: slice) -> Version:
-        ...
+    def __getitem__(self, idx: slice) -> Version: ...
 
     def __getitem__(self, idx: int | slice) -> VersionBit | Version:
         if isinstance(idx, slice):
