@@ -101,6 +101,12 @@ class Config(MutableMapping[str, str]):
             True,
             env_var="PDM_CACHE_DIR",
         ),
+        "log_dir": ConfigItem(
+            "The root directory of log files",
+            platformdirs.user_log_dir("pdm"),
+            True,
+            env_var="PDM_LOG_DIR",
+        ),
         "check_update": ConfigItem(
             "Check if there is any newer version available",
             True,
@@ -115,6 +121,9 @@ class Config(MutableMapping[str, str]):
             "PDM_BUILD_ISOLATION",
             ensure_boolean,
         ),
+        "request_timeout": ConfigItem(
+            "The timeout for network requests in seconds", 15, True, "PDM_REQUEST_TIMEOUT", coerce=int
+        ),
         "global_project.fallback": ConfigItem(
             "Use the global project implicitly if no local project is found",
             False,
@@ -156,7 +165,7 @@ class Config(MutableMapping[str, str]):
             coerce=ensure_boolean,
         ),
         "install.cache_method": ConfigItem(
-            "Specify how to create links to the caches(`symlink/symlink_individual/hardlink/pth`)",
+            "Specify how to create links to the caches(`symlink/hardlink`)",
             "symlink",
         ),
         "python.providers": ConfigItem(
@@ -166,6 +175,11 @@ class Config(MutableMapping[str, str]):
         "python.use_venv": ConfigItem(
             "Use virtual environments when available", True, env_var="PDM_USE_VENV", coerce=ensure_boolean
         ),
+        "python.install_root": ConfigItem(
+            "The root directory to install python interpreters",
+            global_only=True,
+            default=os.path.join(platformdirs.user_data_dir("pdm"), "python"),
+        ),
         "pypi.url": ConfigItem(
             "The URL of PyPI mirror, defaults to https://pypi.org/simple",
             DEFAULT_PYPI_INDEX,
