@@ -17,6 +17,7 @@ from pdm import termui
 from pdm._types import RepositoryConfig
 from pdm.exceptions import NoPythonVersion, PdmUsageError, ProjectError
 from pdm.models.backends import BuildBackend, get_backend_by_spec
+from pdm.models.caches import PackageCache
 from pdm.models.python import PythonInfo
 from pdm.models.repositories import BaseRepository, LockedRepository
 from pdm.models.requirements import Requirement, parse_requirement, strip_extras
@@ -76,12 +77,13 @@ class Project:
         self._lockfile: Lockfile | None = None
         self._environment: BaseEnvironment | None = None
         self._python: PythonInfo | None = None
+        self._cache_dir: Path | None = None
         self.core = core
 
         if global_config is None:
             global_config = platformdirs.user_config_path("pdm") / "config.toml"
         self.global_config = Config(Path(global_config), is_global=True)
-        global_project = Path(self.global_config["global_project.path"])
+        global_project = Path(self.global_config["global_project.path"]).expanduser()
 
         if root_path is None:
             root_path = find_project_root() if not is_global else global_project
@@ -234,15 +236,18 @@ class Project:
                     return python
 
             if not self.root.joinpath("__pypackages__").exists():
+                self.core.ui.warn(
+                    f"Project requires a python version of {self.python_requires}, "
+                    f"The virtualenv is being created for you as it cannot be matched to the right version."
+                )
                 note("python.use_venv is on, creating a virtualenv for this project...")
                 venv_path = self._create_virtualenv()
                 self.python = PythonInfo.from_path(get_venv_python(venv_path))
                 return self.python
 
-        for py_version in self.find_interpreters():
-            if match_version(py_version):
-                if config.get("python.use_venv"):
-                    note("[success]__pypackages__[/] is detected, using the PEP 582 mode")
+        if self.root.joinpath("__pypackages__").exists() or not config["python.use_venv"] or self.is_global:
+            for py_version in self.iter_interpreters(filter_func=match_version):
+                note("[success]__pypackages__[/] is detected, using the PEP 582 mode")
                 self.python = py_version
                 return py_version
 
@@ -266,11 +271,11 @@ class Project:
             else PythonLocalEnvironment(self)
         )
 
-    def _create_virtualenv(self) -> Path:
+    def _create_virtualenv(self, python: str | None = None) -> Path:
         from pdm.cli.commands.venv.backends import BACKENDS
 
         backend: str = self.config["venv.backend"]
-        venv_backend = BACKENDS[backend](self, None)
+        venv_backend = BACKENDS[backend](self, python)
         path = venv_backend.create(
             force=True,
             in_project=self.config["venv.in_project"],
@@ -582,7 +587,13 @@ class Project:
 
     @property
     def cache_dir(self) -> Path:
-        return Path(self.config.get("cache_dir", ""))
+        if self._cache_dir is None:
+            self._cache_dir = Path(self.config.get("cache_dir", "")).expanduser()
+        return self._cache_dir
+
+    @cache_dir.setter
+    def cache_dir(self, value: Path) -> None:
+        self._cache_dir = value
 
     def cache(self, name: str) -> Path:
         path = self.cache_dir / name
@@ -598,6 +609,10 @@ class Project:
 
         return get_wheel_cache(self.cache("wheels"))
 
+    @property
+    def package_cache(self) -> PackageCache:
+        return PackageCache(self.cache("packages"))
+
     def make_candidate_info_cache(self) -> CandidateInfoCache:
         from pdm.models.caches import CandidateInfoCache
 
@@ -610,7 +625,50 @@ class Project:
 
         return HashCache(directory=self.cache("hashes"))
 
-    def find_interpreters(self, python_spec: str | None = None) -> Iterable[PythonInfo]:
+    def iter_interpreters(
+        self,
+        python_spec: str | None = None,
+        search_venv: bool | None = None,
+        filter_func: Callable[[PythonInfo], bool] | None = None,
+    ) -> Iterable[PythonInfo]:
+        """Iterate over all interpreters that matches the given specifier.
+        And optionally install the interpreter if not found.
+        """
+        from pbs_installer._versions import PYTHON_VERSIONS, PythonVersion
+
+        from pdm.cli.commands.python import InstallCommand
+
+        found = False
+        for interpreter in self.find_interpreters(python_spec, search_venv):
+            if filter_func is None or filter_func(interpreter):
+                found = True
+                yield interpreter
+        if found or self.is_global:
+            return
+
+        def get_version(version: PythonVersion) -> str:
+            return f"{version.major}.{version.minor}.{version.micro}"
+
+        if not python_spec:  # handle both empty string and None
+            # Get the best match meeting the requires-python
+            best_match = next((v for v in PYTHON_VERSIONS if get_version(v) in self.python_requires), None)
+            if best_match is None:
+                return
+            python_spec = str(best_match)
+
+        try:
+            # otherwise if no interpreter is found, try to install it
+            installed = InstallCommand.install_python(self, python_spec)
+        except Exception as e:
+            self.core.ui.error(f"Failed to install Python {python_spec}: {e}")
+            return
+        else:
+            if filter_func is None or filter_func(installed):
+                yield installed
+
+    def find_interpreters(
+        self, python_spec: str | None = None, search_venv: bool | None = None
+    ) -> Iterable[PythonInfo]:
         """Return an iterable of interpreter paths that matches the given specifier,
         which can be:
             1. a version specifier like 3.7
@@ -648,7 +706,9 @@ class Project:
                         yield PythonInfo.from_path(python)
                         return
             finder_arg = python_spec
-        finder = self._get_python_finder()
+        if search_venv is None:
+            search_venv = config["python.use_venv"]
+        finder = self._get_python_finder(search_venv)
         for entry in finder.find_all(finder_arg, allow_prereleases=True):
             yield PythonInfo(entry)
         if not python_spec:
@@ -656,14 +716,22 @@ class Project:
             this_python = getattr(sys, "_base_executable", sys.executable)
             yield PythonInfo.from_path(this_python)
 
-    def _get_python_finder(self) -> Finder:
+    def _get_python_finder(self, search_venv: bool = True) -> Finder:
         from findpython import Finder
 
         from pdm.cli.commands.venv.utils import VenvProvider
 
         providers: list[str] = self.config["python.providers"]
-        finder = Finder(resolve_symlinks=True, selected_providers=providers or None)
-        if self.config["python.use_venv"] and (not providers or "venv" in providers):
+        old_rye_root = os.getenv("RYE_PY_ROOT")
+        os.environ["RYE_PY_ROOT"] = os.path.expanduser(self.config["python.install_root"])
+        try:
+            finder = Finder(resolve_symlinks=True, selected_providers=providers or None)
+        finally:
+            if old_rye_root:  # pragma: no cover
+                os.environ["RYE_PY_ROOT"] = old_rye_root
+            else:
+                del os.environ["RYE_PY_ROOT"]
+        if search_venv and (not providers or "venv" in providers):
             venv_pos = providers.index("venv") if providers else 0
             finder.add_provider(VenvProvider(self), venv_pos)
         return finder
