@@ -2,13 +2,13 @@ from __future__ import annotations
 
 import hashlib
 import json
-from typing import Mapping
+import warnings
+from typing import Any, Mapping
 
 from tomlkit import TOMLDocument, items
 
 from pdm import termui
 from pdm.project.toml_file import TOMLBase
-from pdm.utils import deprecation_warning
 
 
 def _remove_empty_tables(doc: dict) -> None:
@@ -39,7 +39,8 @@ class PyProject(TOMLBase):
 
     def write(self, show_message: bool = True) -> None:
         """Write the TOMLDocument to the file."""
-        _remove_empty_tables(self._data)
+        _remove_empty_tables(self._data.get("project", {}))
+        _remove_empty_tables(self._data.get("tool", {}).get("pdm", {}))
         super().write()
         if show_message:
             self.ui.echo("Changes are written to [success]pyproject.toml[/].", verbosity=termui.Verbosity.NORMAL)
@@ -61,19 +62,22 @@ class PyProject(TOMLBase):
         return self._data.get("build-system", {})
 
     @property
-    def resolution_overrides(self) -> Mapping[str, str]:
+    def resolution(self) -> Mapping[str, Any]:
         """A compatible getter method for the resolution overrides
         in the pyproject.toml file.
         """
-        settings = self.settings
-        if "overrides" in settings:
-            deprecation_warning(
-                "The 'tool.pdm.overrides' table has been renamed to "
-                "'tool.pdm.resolution.overrides', please update the "
-                "setting accordingly."
+        return self.settings.get("resolution", {})
+
+    @property
+    def allow_prereleases(self) -> bool | None:
+        if "allow_prereleases" in self.settings:  # pragma: no cover
+            warnings.warn(
+                "'tool.pdm.allow_prereleases' is deprecated, use 'tool.pdm.resolution.allow-prereleases' instead.",
+                FutureWarning,
+                stacklevel=3,
             )
-            return settings["overrides"]
-        return settings.get("resolution", {}).get("overrides", {})
+            return self.settings["allow_prereleases"]
+        return self.resolution.get("allow-prereleases")
 
     def content_hash(self, algo: str = "sha256") -> str:
         """Generate a hash of the sensible content of the pyproject.toml file.
@@ -85,7 +89,7 @@ class PyProject(TOMLBase):
             "dev-dependencies": self.settings.get("dev-dependencies", {}),
             "optional-dependencies": self.metadata.get("optional-dependencies", {}),
             "requires-python": self.metadata.get("requires-python", ""),
-            "overrides": self.resolution_overrides,
+            "overrides": self.resolution.get("overrides", {}),
         }
         pyproject_content = json.dumps(dump_data, sort_keys=True)
         hasher = hashlib.new(algo)
