@@ -2,10 +2,10 @@ from __future__ import annotations
 
 import inspect
 import os
-from typing import TYPE_CHECKING, Callable, cast
+from typing import TYPE_CHECKING, Callable
 
 from packaging.specifiers import InvalidSpecifier, SpecifierSet
-from packaging.version import InvalidVersion, Version
+from packaging.version import InvalidVersion
 from resolvelib import AbstractProvider, RequirementsConflicted
 from resolvelib.resolvers import Criterion
 
@@ -15,7 +15,7 @@ from pdm.models.repositories import LockedRepository
 from pdm.models.requirements import FileRequirement, parse_requirement, strip_extras
 from pdm.resolver.python import PythonCandidate, PythonRequirement, find_python_matches, is_python_satisfied_by
 from pdm.termui import logger
-from pdm.utils import deprecation_warning, is_url, normalize_name, url_without_fragments
+from pdm.utils import deprecation_warning, is_url, normalize_name, parse_version, url_without_fragments
 
 if TYPE_CHECKING:
     from typing import Any, Iterable, Iterator, Mapping, Sequence
@@ -101,12 +101,8 @@ class BaseProvider(AbstractProvider):
         editable = requirement.editable
         is_named = requirement.is_named
         is_pinned = requirement.is_pinned
-        is_prerelease = (
-            bool(requirement.prerelease)
-            or requirement.specifier is not None
-            and bool(requirement.specifier.prereleases)
-        )
-        specifier_parts = len(requirement.specifier) if requirement.specifier else 0
+        is_prerelease = bool(requirement.prerelease) and bool(requirement.specifier.prereleases)
+        specifier_parts = len(requirement.specifier)
         return (not editable, is_named, not is_pinned, not is_prerelease, -specifier_parts)
 
     def identify(self, requirement_or_candidate: Requirement | Candidate) -> str:
@@ -143,9 +139,7 @@ class BaseProvider(AbstractProvider):
         self._known_depth[self.identify(candidate)] = dep_depth
         is_backtrack_cause = any(dep.identify() in backtrack_identifiers for dep in deps)
         is_file_or_url = any(not requirement.is_named for requirement, _ in information[identifier])
-        operators = [
-            spec.operator for req, _ in information[identifier] if req.specifier is not None for spec in req.specifier
-        ]
+        operators = [spec.operator for req, _ in information[identifier] for spec in req.specifier]
         is_python = identifier == "python"
         is_pinned = any(op[:2] == "==" for op in operators)
         constraints = len(operators)
@@ -193,7 +187,7 @@ class BaseProvider(AbstractProvider):
                 candidate = self.locked_candidates[key]
                 if candidate.version is not None:
                     try:
-                        parsed_version = Version(candidate.version)
+                        parsed_version = parse_version(candidate.version)
                     except InvalidVersion:  # pragma: no cover
                         pass
                     else:
@@ -218,6 +212,8 @@ class BaseProvider(AbstractProvider):
                 return (c for c in candidates if c not in incompat)
             elif identifier in self.overrides:
                 return iter(self.get_override_candidates(identifier))
+            elif (name := strip_extras(identifier)[0]) in self.overrides:
+                return iter(self.get_override_candidates(name))
             reqs = sorted(requirements[identifier], key=self.requirement_preference)
             if not reqs:
                 return iter(())
@@ -250,7 +246,7 @@ class BaseProvider(AbstractProvider):
     def is_satisfied_by(self, requirement: Requirement, candidate: Candidate) -> bool:
         if isinstance(requirement, PythonRequirement):
             return is_python_satisfied_by(requirement, candidate)
-        elif candidate.identify() in self.overrides:
+        elif (name := candidate.identify()) in self.overrides or strip_extras(name)[0] in self.overrides:
             return True
         if not requirement.is_named:
             if candidate.req.is_named:
@@ -267,7 +263,7 @@ class BaseProvider(AbstractProvider):
         # Allow prereleases if: 1) it is not specified in the tool settings or
         # 2) the candidate doesn't come from PyPI index.
         allow_prereleases = self.allow_prereleases in (True, None) or not candidate.req.is_named
-        return cast(SpecifierSet, requirement.specifier).contains(version, allow_prereleases)
+        return requirement.specifier.contains(version, allow_prereleases)
 
     def get_dependencies(self, candidate: Candidate) -> list[Requirement]:
         if isinstance(candidate, PythonCandidate):
@@ -289,7 +285,7 @@ class BaseProvider(AbstractProvider):
                 & requires_python
                 & candidate.req.requires_python
                 & self.repository.environment.python_requires
-            ).is_impossible:
+            ).is_empty():
                 continue
             if dep.identify() in self.excludes:
                 continue
@@ -305,7 +301,7 @@ class BaseProvider(AbstractProvider):
         new_requires_python = candidate.req.requires_python & self.repository.environment.python_requires
         if not (
             candidate.identify() in self.overrides
-            or new_requires_python.is_impossible
+            or new_requires_python.is_empty()
             or requires_python.is_superset(new_requires_python)
         ):
             valid_deps.append(PythonRequirement.from_pyspec_set(requires_python))
